package sun.security.pkcs10;

import java.util.Arrays;
import java.util.Base64;
import java.io.PrintStream;
import java.security.cert.CertificateException;
import java.security.AlgorithmParameters;
import sun.security.util.DerOutputStream;
import java.security.NoSuchAlgorithmException;
import java.io.IOException;
import sun.security.util.DerValue;
import java.security.ProviderException;
import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.SignatureException;
import sun.security.util.SignatureUtil;
import java.security.Signature;
import sun.security.x509.X509Key;
import java.math.BigInteger;
import sun.security.x509.AlgorithmId;
import sun.security.util.DerInputStream;
import java.security.PublicKey;
import sun.security.x509.X500Name;

public class PKCS10
{
    private X500Name subject;
    private PublicKey subjectPublicKeyInfo;
    private String sigAlg;
    private PKCS10Attributes attributeSet;
    private byte[] encoded;
    
    public PKCS10(PublicKey subjectPublicKeyInfo) {
        this.subjectPublicKeyInfo = subjectPublicKeyInfo;
        this.attributeSet = new PKCS10Attributes();
    }
    
    public PKCS10(PublicKey subjectPublicKeyInfo, PKCS10Attributes attributeSet) {
        this.subjectPublicKeyInfo = subjectPublicKeyInfo;
        this.attributeSet = attributeSet;
    }
    
    public PKCS10(byte[] byteArray) throws IOException, SignatureException, NoSuchAlgorithmException {
        this.encoded = byteArray;
        DerValue[] sequence = new DerInputStream(byteArray).getSequence(3);
        if (sequence.length != 3) {
            throw new IllegalArgumentException("not a PKCS #10 request");
        }
        byteArray = sequence[0].toByteArray();
        AlgorithmId parse = AlgorithmId.parse(sequence[1]);
        byte[] bitString = sequence[2].getBitString();
        if (!sequence[0].data.getBigInteger().equals(BigInteger.ZERO)) {
            throw new IllegalArgumentException("not PKCS #10 v1");
        }
        this.subject = new X500Name(sequence[0].data);
        this.subjectPublicKeyInfo = X509Key.parse(sequence[0].data.getDerValue());
        if (sequence[0].data.available() != 0) {
            this.attributeSet = new PKCS10Attributes(sequence[0].data);
        }
        else {
            this.attributeSet = new PKCS10Attributes();
        }
        if (sequence[0].data.available() != 0) {
            throw new IllegalArgumentException("illegal PKCS #10 data");
        }
        try {
            this.sigAlg = parse.getName();
            Signature instance = Signature.getInstance(this.sigAlg);
            SignatureUtil.initVerifyWithParam(instance, this.subjectPublicKeyInfo, SignatureUtil.getParamSpec(this.sigAlg, parse.getParameters()));
            instance.update(byteArray);
            if (!instance.verify(bitString)) {
                throw new SignatureException("Invalid PKCS #10 signature");
            }
        }
        catch (InvalidKeyException ex) {
            throw new SignatureException("Invalid key");
        }
        catch (InvalidAlgorithmParameterException cause) {
            throw new SignatureException("Invalid signature parameters", cause);
        }
        catch (ProviderException ex2) {
            throw new SignatureException("Error parsing signature parameters", ex2.getCause());
        }
        return;
    }
    
    public void encodeAndSign(X500Name subject, Signature signature) throws CertificateException, IOException, SignatureException {
        if (this.encoded != null) {
            throw new SignatureException("request is already signed");
        }
        this.subject = subject;
        DerOutputStream out = new DerOutputStream();
        out.putInteger(BigInteger.ZERO);
        subject.encode(out);
        out.write(this.subjectPublicKeyInfo.getEncoded());
        this.attributeSet.encode(out);
        DerOutputStream derOutputStream = new DerOutputStream();
        derOutputStream.write((byte)48, out);
        byte[] byteArray = derOutputStream.toByteArray();
        DerOutputStream out2 = derOutputStream;
        signature.update(byteArray, 0, byteArray.length);
        byte[] sign = signature.sign();
        this.sigAlg = signature.getAlgorithm();
        AlgorithmId algorithmId;
        try {
            AlgorithmParameters parameters = signature.getParameters();
            algorithmId = ((parameters == null) ? AlgorithmId.get(signature.getAlgorithm()) : AlgorithmId.get(parameters));
        }
        catch (NoSuchAlgorithmException cause) {
            throw new SignatureException(cause);
        }
        algorithmId.encode(out2);
        out2.putBitString(sign);
        DerOutputStream derOutputStream2 = new DerOutputStream();
        derOutputStream2.write((byte)48, out2);
        this.encoded = derOutputStream2.toByteArray();
        return;
    }
    
    public X500Name getSubjectName() {
        return this.subject;
    }
    
    public PublicKey getSubjectPublicKeyInfo() {
        return this.subjectPublicKeyInfo;
    }
    
    public String getSigAlg() {
        return this.sigAlg;
    }
    
    public PKCS10Attributes getAttributes() {
        return this.attributeSet;
    }
    
    public byte[] getEncoded() {
        if (this.encoded != null) {
            return (byte[])this.encoded.clone();
        }
        return null;
    }
    
    public void print(PrintStream printStream) throws IOException, SignatureException {
        if (this.encoded == null) {
            throw new SignatureException("Cert request was not signed");
        }
        byte[] lineSeparator = { 13, 10 };
        printStream.println("-----BEGIN NEW CERTIFICATE REQUEST-----");
        printStream.println(Base64.getMimeEncoder(64, lineSeparator).encodeToString(this.encoded));
        printStream.println("-----END NEW CERTIFICATE REQUEST-----");
        return;
    }
    
    @Override
    public String toString() {
        return "[PKCS #10 certificate request:\n" + this.subjectPublicKeyInfo.toString() + " subject: <" + this.subject + ">\n attributes: " + this.attributeSet.toString() + "\n]";
    }
    
    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (!(o instanceof PKCS10)) {
            return false;
        }
        if (this.encoded == null) {
            return false;
        }
        byte[] encoded = ((PKCS10)o).getEncoded();
        return encoded != null && Arrays.equals(this.encoded, encoded);
    }
    
    @Override
    public int hashCode() {
        int n = 0;
        if (this.encoded != null) {
            for (int i = 1; i < this.encoded.length; ++i) {
                n += this.encoded[i] * i;
            }
        }
        return n;
    }
}
